import os
import pickle
import numpy

import matplotlib 
matplotlib.rc('xtick', labelsize=22) 
matplotlib.rc('ytick', labelsize=22) 
matplotlib.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
matplotlib.rc('text', usetex=True)

sizes=[10000,100000,1000000,10000000,50000000]
labelsize=22
titlesize=21
linewidth=3
markersize=8
from pylab import *

fnames=[('results/wds_linadd22_shift0.3_%d.dump', 'SVM$^{light}$+linadd WDS'),
		('results/wd_linadd_%d.dump', 'SVM$^{light}$+linadd $d=8$'),
		('results/wd_linadd20_%d.dump', 'SVM$^{light}$+linadd $d=20$'),
		('results/wd_ocas_%d.dump', 'COFFIN OC $d=8$ (dense)'),
	('results/wd_hash12_ocas_%d.dump', 'COFFIN OC $d=20$, $h=12$'),
	#('results/wd_hash16_ocas_%d.dump', 'COFFIN OC $d=20$, $h=16$'),
	#('results/wd_hash12_liblinear_primal_%d.dump', 'LiblinearL2$_p$ $d=20$, $h=12$'),
	#('results/wd_hash12_liblinear_dual_%d.dump', 'LiblinearL2$_d$ $d=20$, $h=12$'),
	('results/wd_hash12_liblinear_dual_l1_%d.dump', 'COFFIN LL$_d$ $d=20$, $h=12$'),
	('results/wd_hash16_liblinear_dual_l1_%d.dump', 'COFFIN LL$_d$ $d=20$, $h=16$'),
	#('results/wd_hash12_sgd_l1_%d.dump', 'SGD$ $d=20$, $h=12$'),
	]


title_line='%40.40s' % 'method / sample size &'
for num in [10000,100000,1000000,10000000,50000000]:
	s='\\multicolumn{2}{|c|}{' + '%.0e' % num + '}' + '&'
	#title_line+='%40.40s '  % s
	title_line+=s

methods=[]
all_timings=-numpy.ones((5,len(fnames)))
all_prcs=-numpy.ones((5,len(fnames)))
idx1=0
body=dict()
for f in fnames:
	body[f]='%40.40s &' % f[1]
	methods.append('%s' % f[1])

for num in sizes:
	#body[f]+='%d\t' % num
	idx2=0
	for f in fnames:
		fname=f[0] % num
		if os.path.exists(fname):
			(result,elapsed)=pickle.load(file(fname))
			body[f]+='%20.20s &' % ('%ds & %2.2f\\%%' % (elapsed,100*result['auPRC']))
			all_prcs[idx1,idx2]=100*result['auPRC']
			all_timings[idx1,idx2]=elapsed
		else:
			body[f]+='%20.20s' % '\\multicolumn{2}{|c|}' + '{-}' + '&'
		idx2+=1
	idx1+=1

print title_line[:-1] + '\\\\\hline'
for f in fnames:
	print body[f][:-1] + '\\\\'
print '\\hline'

styles=['o-','o-','o-',
        'D-','D-','D-',
		'^-','^-']

colors=[ cm.jet(i) for i in arange(0,1,0.12) ]
#colors=[ cm.hot(i) for i in arange(0,1,0.1) ]
#colors=['#FF0000','#A00000','#500000',
#'#000050','#0000A0','#0000FF',
#'#000000','#999999']

#              SVM$^{light}$+linadd $d=8$ &        65s & 7.14\% &     1114s & 26.37\% &    24861s & 43.80\% &  2112112s & 53.01\% &\multicolumn{2}{|c|}{-}\\
#              SVM$^{light}$+linadd $d=8$ &       57s & 10.37\% &      970s & 28.62\% &    34110s & 43.78\% &\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}\\
#             SVM$^{light}$+linadd $d=20$ &       56s & 11.15\% &     1033s & 31.80\% &    34586s & 46.27\% &\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}\\
#                      Ocas $d=8$ (dense) &      167s & 10.00\% &      948s & 28.57\% &     9952s & 43.84\% &   131202s & 53.26\% &   909105s & 57.78\% \\
#                     Ocas $d=20$, $h=12$ &       65s & 10.81\% &      435s & 31.80\% &     5349s & 46.12\% &    76311s & 54.31\% &   908654s & 57.89\% \\
#                     Ocas $d=20$, $h=16$ &      363s & 10.66\% &     1430s & 31.90\% &    10288s & 46.43\% &\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}\\
#          LiblinearL2$_p$ $d=20$, $h=12$ &      784s & 10.29\% &     6985s & 31.57\% &\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}\\
#          LiblinearL2$_d$ $d=20$, $h=12$ &       63s & 10.00\% &      445s & 31.34\% &     2938s & 45.38\% &\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}\\
#          LiblinearL1$_d$ $d=20$, $h=12$ &       61s & 10.59\% &      360s & 31.59\% &     3783s & 45.97\% &    25902s & 54.17\% &   132581s & 57.75\% \\
#          LiblinearL1$_d$ $d=20$, $h=16$ &      111s & 10.52\% &      604s & 31.69\% &     4590s & 46.26\% &    44232s & 54.56\% &   247907s & 58.57\% \\
#                     SGD$ $d=20$, $h=12$ &\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}&\multicolumn{2}{|c|}{-}\\


figure()
for j in range(len(fnames))[::-1]:
	#if all_timings[i,j]<0 or all_prcs[i,j]<0:
	#	continue
	semilogx(all_timings[:,j], all_prcs[:,j], styles[j], linewidth=linewidth, markersize=markersize, color=colors[j])
	hold('on')
grid('on')
t=xlabel("Training time in Seconds (logarithmic)")
setp(t, fontsize=labelsize-2)
t=ylabel("auPRC")
setp(t, fontsize=labelsize-2)
print methods
legend(methods[::-1], loc='lower right')
#x=axis()
#x[2]=
axis('tight')
#import pdb
#pdb.set_trace()
savefig('results/time_vs_auPRC.pdf')
#show()

figure()
for j in range(len(fnames))[::-1]:
	#if all_timings[i,j]<0 or all_prcs[i,j]<0:
	#	continue
	loglog(sizes[0:len(all_timings[:,j])], all_timings[:,j], styles[j], linewidth=linewidth, markersize=markersize, color=colors[j])
	hold('on')
grid('on')
t=ylabel("Training time in Seconds (logarithmic)")
setp(t, fontsize=labelsize-2)
t=xlabel("Data Set Size (logarithmic)")
setp(t, fontsize=labelsize-2)
print methods
legend(methods[::-1], loc='upper left')
#x=axis()
#x[2]=
axis('tight')
#import pdb
#pdb.set_trace()
savefig('results/time_vs_auPRC.pdf')
#show()

##################################################################################
print
print
fnames=[('results/webspam_light_%d.dump', 'SVM$^{light}$+Sparse Poly Kernel'),
		('results/webspam_hash20_ocas_%d.dump', 'Ocas + COFIS $d=2$'),
	]


title_line='%40.40s' % 'method / sample size &'
for num in [1000,10000,100000,350000]:
	s='\\multicolumn{2}{|c|}{' + '%.0e' % num + '}' + '&'
	#title_line+='%40.40s '  % s
	title_line+=s

body=dict()
for f in fnames:
	body[f]='%40.40s &' % f[1]

for num in [1000,10000,100000,350000]:
	#body[f]+='%d\t' % num
	for f in fnames:
		fname=f[0] % num
		if os.path.exists(fname):
			(result,elapsed)=pickle.load(file(fname))
			body[f]+='%20.20s &' % ('%ds & %2.2f\\%%' % (elapsed,100*result['auPRC']))
			pass
		else:
			body[f]+='%20.20s' % '\\multicolumn{2}{|c|}' + '{-}' + '&'

print title_line[:-1] + '\\\\\hline'
for f in fnames:
	print body[f][:-1] + '\\\\'
print '\\hline'

##################################################################################
print
print
fnames=[('results/wd_hash%d_ocas_100000.dump', 'WD Ocas')]

title_line='%10.10s' % ' '
for num in xrange(1,18):
	title_line+='%10.10s bit'  % `num`

body=dict()
for f in fnames:
	body[f]='%14.14s' % f[1]

bits=[]
timings=[]
prcs=[]
for num in xrange(1,18):
	#body[f]+='%d\t' % num
	for f in fnames:
		fname=f[0] % num
		if os.path.exists(fname):
			(result,elapsed)=pickle.load(file(fname))
			body[f]+='%14.14s' % ('%ds %2.2f%%' % (elapsed,100*result['auPRC']))
			bits.append(num)
			timings.append(elapsed)
			prcs.append(result['auPRC'])
			pass
		else:
			body[f]+='%14.14s' % '-'

print title_line
for f in fnames:
	print body[f]
print
print

f=figure()
ax=f.axes
bar(bits,timings)
t=title('Number of Bits vs. Training Time on 100k examples')
setp(t, fontsize=titlesize)
t=ylabel('Training Time')
setp(t, fontsize=labelsize)
t=xlabel('Number of Bits')
setp(t, fontsize=labelsize)
axis('tight')

#for t in get_xticklabels():
#	t.set_fontsize(ticksz)
#for t in get_yticklabels():
#	t.set_fontsize(ticksz)
savefig('results/num_bits_vs_time100k.pdf')

figure()
bar(bits,prcs)
t=title('Number of Bits vs. auPRC on 100k examples')
setp(t, fontsize=titlesize)
t=ylabel('auPRC')
setp(t, fontsize=labelsize)
t=xlabel('Number of Bits')
setp(t, fontsize=labelsize)
axis('tight')
savefig('results/num_bits_vs_auPRC100k.pdf')
#show()
