num=10000
degree=8
C=1
epsilon=1e-2
num_threads=16
order=8
gap=0
reverse=True
normalize=True
use_bias=True

from shogun.Features import StringCharFeatures, StringByteFeatures, \
		StringWordFeatures, CombinedDotFeatures, \
		WDFeatures, ImplicitWeightedSpecFeatures, Labels, DNA, RAWDNA
from shogun.PreProc import SortWordString
from shogun.Classifier import LibLinear, SVMSGD, SVMOcas
from shogun.Library import MSG_INFO, MSG_DEBUG

import evaluation
import numpy
import scipy
import pickle
import time
import os.path

def read_lab(fname):
	x=file(fname).readlines()
	return numpy.array([ float(i) for i in x ])

def train_wdocas(num):
	train_filename='data/H_sapiens_acc_all_examples_plain_139-279_' + `num` + '.fasta'
	left_train_filename=train_filename + '_up'
	right_train_filename=train_filename + '_down'
	label_train_filename='data/H_sapiens_acc_all_examples_plain_' +`num` +'.label'

	test_filename='data/H_sapiens_acc_all_examples_plain_139-279_5e7_test.fasta';
	left_test_filename=test_filename + '_up'
	right_test_filename=test_filename + '_down'
	label_test_filename='data/H_sapiens_acc_all_examples_plain_5e7_test.label'

	#test_filename=train_filename
	#left_test_filename=left_train_filename
	#right_test_filename=right_train_filename
	#label_test_filename=label_train_filename

	labels_train=read_lab(label_train_filename)
	labels_test=read_lab(label_test_filename)

	# spec left
	feats_train_left_char=StringCharFeatures(DNA)
	feats_train_left_char.io.set_loglevel(MSG_DEBUG)
	feats_train_left_char.load_dna_file(left_train_filename, False)
	feats_train_left_word=StringWordFeatures(feats_train_left_char.get_alphabet())
	feats_train_left_word.obtain_from_char(feats_train_left_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train_left_word)
	feats_train_left_word.add_preproc(preproc)
	feats_train_left_word.apply_preproc()
	feats_train_left=ImplicitWeightedSpecFeatures(feats_train_left_word, normalize);

	feats_test_left_char=StringCharFeatures(DNA)
	feats_test_left_char.load_dna_file(left_test_filename, False)
	feats_test_left_word=StringWordFeatures(feats_test_left_char.get_alphabet())
	feats_test_left_word.obtain_from_char(feats_test_left_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_test_left_word)
	feats_test_left_word.add_preproc(preproc)
	feats_test_left_word.apply_preproc()
	feats_test_left=ImplicitWeightedSpecFeatures(feats_test_left_word, normalize);

	# wd center
	feats_train_center_raw=StringByteFeatures(DNA)
	feats_train_center_raw.load_dna_file(train_filename)
	feats_train_center=WDFeatures(feats_train_center_raw, degree, degree)

	feats_test_center_raw=StringByteFeatures(DNA)
	feats_test_center_raw.load_dna_file(test_filename)
	feats_test_center=WDFeatures(feats_test_center_raw, degree, degree)

	#spec right
	feats_train_right_char=StringCharFeatures(DNA)
	feats_train_right_char.load_dna_file(right_train_filename, False)
	feats_train_right_word=StringWordFeatures(feats_train_right_char.get_alphabet())
	feats_train_right_word.obtain_from_char(feats_train_right_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train_right_word)
	feats_train_right_word.add_preproc(preproc)
	feats_train_right_word.apply_preproc()
	feats_train_right=ImplicitWeightedSpecFeatures(feats_train_right_word, normalize);

	feats_test_right_char=StringCharFeatures(DNA)
	feats_test_right_char.load_dna_file(right_test_filename, False)
	feats_test_right_word=StringWordFeatures(feats_test_right_char.get_alphabet())
	feats_test_right_word.obtain_from_char(feats_test_right_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_test_right_word)
	feats_test_right_word.add_preproc(preproc)
	feats_test_right_word.apply_preproc()
	feats_test_right=ImplicitWeightedSpecFeatures(feats_test_right_word, normalize);

	# features
	feats_train=CombinedDotFeatures()
	feats_train.append_feature_obj(feats_train_left)
	feats_train.append_feature_obj(feats_train_center)
	feats_train.append_feature_obj(feats_train_right)

	feats_test=CombinedDotFeatures()
	feats_test.append_feature_obj(feats_test_left)
	feats_test.append_feature_obj(feats_test_center)
	feats_test.append_feature_obj(feats_test_right)

	# train svm
	svm=SVMOcas(C, feats_train, Labels(labels_train))
	svm.set_epsilon(epsilon)
	svm.io.enable_progress()
	svm.parallel.set_num_threads(num_threads)
	start = time.clock()
	svm.set_bias_enabled(use_bias)

	svm.train()
	elapsed = (time.clock() - start)
	out=svm.classify(feats_test).get_labels()

	return (evaluation.compute_scores(out, labels_test), elapsed)

for num in [10000,100000,1000000,10000000,50000000]:
	fname='results/wd_ocas_%d.dump' % num
	if os.path.exists(fname):
		continue
	print num
	(result,elapsed)=train_wdocas(num)
	print elapsed,result
	pickle.dump((result,elapsed), file(fname, 'w'))
