degree=8
C=1
epsilon=1e-3
num_threads=1
order=8
gap=0
reverse=True
use_sign=False
use_bias=True

from shogun.Features import StringCharFeatures, StringWordFeatures,\
		CombinedFeatures, CombinedDotFeatures, WDFeatures, \
		ImplicitWeightedSpecFeatures, Labels, DNA
from shogun.Kernel import WeightedDegreeStringKernel, \
		WeightedCommWordStringKernel, CombinedKernel
from shogun.PreProc import SortWordString
from shogun.Library import MSG_INFO, MSG_DEBUG

from shogun.Classifier import SVMLight, LibLinear, SVMSGD, SVMOcas
import evaluation
import numpy
import scipy
import pickle
import time
import os.path

def read_lab(fname):
	x=file(fname).readlines()
	return numpy.array([ float(i) for i in x ])

def train_linadd(num):
	train_filename='data/H_sapiens_acc_all_examples_plain_139-279_' + `num` + '.fasta'
	left_train_filename=train_filename + '_up'
	right_train_filename=train_filename + '_down'
	label_train_filename='data/H_sapiens_acc_all_examples_plain_' +`num` +'.label'

	test_filename='data/H_sapiens_acc_all_examples_plain_139-279_5e7_test.fasta';
	left_test_filename=test_filename + '_up'
	right_test_filename=test_filename + '_down'
	label_test_filename='data/H_sapiens_acc_all_examples_plain_5e7_test.label'

	#test_filename=train_filename
	#left_test_filename=left_train_filename
	#right_test_filename=right_train_filename
	#label_test_filename=label_train_filename

	labels_train=read_lab(label_train_filename)
	labels_test=read_lab(label_test_filename)

	#print num, len(labels_train), len(labels_test)
	# spec left
	sub_kernel_left=WeightedCommWordStringKernel(0, use_sign)
	#sub_kernel_left.io.set_loglevel(MSG_DEBUG)
	sub_kernel_left.io.enable_progress()
	feats_train_left_char=StringCharFeatures(DNA)
	feats_train_left_char.io.enable_progress()
	feats_train_left_char.load_dna_file(left_train_filename, False)
	alpha=feats_train_left_char.get_alphabet()
	#import pdb
	#pdb.set_trace()
	#print alpha.get_num_symbols_in_histogram()
	feats_train_left=StringWordFeatures(feats_train_left_char.get_alphabet())
	feats_train_left.obtain_from_char(feats_train_left_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train_left)
	feats_train_left.add_preproc(preproc)
	feats_train_left.apply_preproc()

	sub_kernel_left=WeightedCommWordStringKernel(0, use_sign)
	feats_test_left_char=StringCharFeatures(DNA)
	feats_test_left_char.load_dna_file(left_test_filename, False)
	feats_test_left=StringWordFeatures(feats_test_left_char.get_alphabet())
	feats_test_left.obtain_from_char(feats_test_left_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_test_left)
	feats_test_left.add_preproc(preproc)
	feats_test_left.apply_preproc()

	# wd center
	sub_kernel_center=WeightedDegreeStringKernel(degree)
	feats_train_center=StringCharFeatures(DNA)
	feats_train_center.load_dna_file(train_filename, False)
	feats_test_center=StringCharFeatures(DNA)
	feats_test_center.load_dna_file(test_filename, False)

	#spec right
	sub_kernel_right=WeightedCommWordStringKernel(0, use_sign)
	feats_train_right_char=StringCharFeatures(DNA)
	feats_train_right_char.load_dna_file(right_train_filename, False)
	feats_train_right=StringWordFeatures(feats_train_right_char.get_alphabet())
	feats_train_right.obtain_from_char(feats_train_right_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train_right)
	feats_train_right.add_preproc(preproc)
	feats_train_right.apply_preproc()

	sub_kernel_right=WeightedCommWordStringKernel(0, use_sign)
	feats_test_right_char=StringCharFeatures(DNA)
	feats_test_right_char.load_dna_file(right_test_filename, False)
	feats_test_right=StringWordFeatures(feats_test_right_char.get_alphabet())
	feats_test_right.obtain_from_char(feats_test_right_char, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_test_right)
	feats_test_right.add_preproc(preproc)
	feats_test_right.apply_preproc()

	# features
	feats_train=CombinedFeatures()
	feats_train.append_feature_obj(feats_train_left)
	feats_train.append_feature_obj(feats_train_center)
	feats_train.append_feature_obj(feats_train_right)

	feats_test=CombinedFeatures()
	feats_test.append_feature_obj(feats_test_left)
	feats_test.append_feature_obj(feats_test_center)
	feats_test.append_feature_obj(feats_test_right)

	# kernel
	kernel=CombinedKernel()
	kernel.append_kernel(sub_kernel_left)
	kernel.append_kernel(sub_kernel_center)
	kernel.append_kernel(sub_kernel_right)

	# train svm w/ linadd
	kernel.init(feats_train, feats_train)
	svm=SVMLight(C, kernel, Labels(labels_train))
	svm.io.enable_progress()
	svm.set_epsilon(epsilon)
	svm.parallel.set_num_threads(num_threads)
	svm.set_bias_enabled(use_bias)
	start = time.clock()
	svm.train()
	elapsed = (time.clock() - start)

	# apply SVM
	kernel.init(feats_train, feats_test)
	out=svm.classify().get_labels()

	return (evaluation.compute_scores(out, labels_test), elapsed)

#degree=8
#for num in [10000,100000,1000000]: #,10000000,50000000]:
#	fname='results/wd_linadd_%d.dump' % num
#	if os.path.exists(fname):
#		continue
#	print num
#	(result,elapsed)=train_linadd(num)
#	print elapsed,result
#	pickle.dump((result,elapsed), file(fname, 'w'))
#
degree=20
epsilon=1e-2
for num in [10000,100000,1000000,10000000]:#,50000000]:
	fname='results/wd_linadd20_%d.dump' % num
	if os.path.exists(fname):
		continue
	print num
	(result,elapsed)=train_linadd(num)
	print elapsed,result
	pickle.dump((result,elapsed), file(fname, 'w'))
