# Optimal settings from http://www.fml.tuebingen.mpg.de/raetsch/suppl/splice
#
# H.sapiens
#
# window	C	order	shift	ppseudo	npseudo	type	method
# 199+[-60,80]	3	22	0	 	 	acceptor	WD-SVM
# 199+[-60,80]	3	22	0.3	 	 	acceptor	WDS-SVM

use_bias=True
degree=22
C=3
epsilon=1e-2
num_threads=16
shift=0.3
lwin=-60
rwin=80

from shogun.Features import StringCharFeatures, Labels, DNA
from shogun.Kernel import WeightedDegreePositionStringKernel
from shogun.Classifier import SVMLight

import evaluation
import numpy
import scipy
import pickle
import time
import os.path

def comp_shifts():
	shifts = numpy.array(numpy.ceil(shift * numpy.abs( numpy.arange(lwin,rwin+1)+1 )), dtype=numpy.int32)
	return shifts

def read_lab(fname):
	x=file(fname).readlines()
	return numpy.array([ float(i) for i in x ])

def train_linadd(num):
	train_filename='data/H_sapiens_acc_all_examples_plain_139-279_' + `num` + '.fasta'
	label_train_filename='data/H_sapiens_acc_all_examples_plain_' +`num` +'.label'

	test_filename='data/H_sapiens_acc_all_examples_plain_139-279_5e7_test.fasta';
	label_test_filename='data/H_sapiens_acc_all_examples_plain_5e7_test.label'

	labels_train=read_lab(label_train_filename)
	labels_test=read_lab(label_test_filename)

	# wd center
	kernel=WeightedDegreePositionStringKernel(0, degree)
	kernel.set_shifts(comp_shifts())
	feats_train=StringCharFeatures(DNA)
	feats_train.load_dna_file(train_filename, False)
	feats_test=StringCharFeatures(DNA)
	feats_test.load_dna_file(test_filename, False)

	# train svm w/ linadd
	kernel.init(feats_train, feats_train)
	svm=SVMLight(C, kernel, Labels(labels_train))
	svm.io.enable_progress()
	svm.set_epsilon(epsilon)
	svm.parallel.set_num_threads(num_threads)
	svm.set_bias_enabled(use_bias)
	start = time.clock()
	svm.train()
	elapsed = (time.clock() - start)

	# apply SVM
	kernel.init(feats_train, feats_test)
	out=svm.classify().get_labels()

	return (evaluation.compute_scores(out, labels_test), elapsed)

for num in [10000,100000,1000000,10000000]:#,50000000]:
	fname='results/wds_linadd22_shift0.3_%d.dump' % num
	if os.path.exists(fname):
		continue
	print num
	(result,elapsed)=train_linadd(num)
	print elapsed,result
	pickle.dump((result,elapsed), file(fname, 'w'))
